/******************************************************************************
                          C++ Interface: VertexCache
                            -------------------
    description       :
    begin             : Mon Mar 28 2005
    author            : Martin Marinov, (C) 2005
    email             : marinov@cs.rwth-aachen.de
    copyright         : See LICENSE.txt file that comes with this distribution
 ******************************************************************************/

#ifndef __VertexCache_h__
#define __VertexCache_h__

#include <assert.h>
#include <vector>
#include <algorithm>

namespace tr
{

class VertexCache
{
public:
  typedef std::vector<int>                  IndexArray;
  typedef IndexArray::const_reverse_iterator ConstIndexIter;
  
private:
  IndexArray                                cache_;

public:
  VertexCache(int _size)
  : cache_(_size, -1)
  {}
  
  inline void                               clear()
  { cache_.assign(cache_.size(), -1); }
  
  inline int                                insert(int _vertex_index)
  {
    int last_vertex_index = cache_.back();
    cache_.pop_back();//free space for the new element
    cache_.insert(cache_.begin(), _vertex_index);
    return last_vertex_index;
  }
  
  inline bool                               is_in(int _vertex_index) const
  { return std::find(cache_.begin(), cache_.end(), _vertex_index) != cache_.end(); }
  
  inline unsigned int                       size() const
  { return cache_.size(); }
  inline bool                               empty() const
  { return cache_.empty(); }
  
  inline int                                last_vertex_index() const
  {
    assert(!cache_.empty());
    for (int i = (int)cache_.size() - 1; i >= 0; --i)
    {
      if (cache_[i] >= 0)
      {
        return cache_[i];
      }
    }
    return -1;
  }
  
  inline int                                operator [] (uint i) const
  { 
    assert(i < cache_.size());
    return cache_[i];
  }  
};

}//tr

#endif//__VertexCache_h__
