//=============================================================================
//
//  CLASS LightMap
//
//=============================================================================


#ifndef PBR_LIGHTMAP_HH
#define PBR_LIGHTMAP_HH


//== INCLUDES =================================================================

#include <OpenMesh/Core/Geometry/VectorT.hh>
#include "globjects.hh"


//== NAMESPACES ===============================================================


//== CLASS DEFINITION =========================================================

/** \class LightMap LightMap.hh <PBR/src/Utils/LightMap.hh>

Stores lighting terms as an OpenGL-Cubemap. Use the Phong-Blinn light model.

*/

class LightMap : public TextureCubeMap
{
public:

  /**
  Constructor.
  \param texunit specifies the texture unit to be used
  \param _mode specifies the format of the lightmap. Either GL_RGBA or 
  GL_LUMINANCE_ALPHA can be used.
  */
  LightMap(GLenum texunit=GL_NONE, GLint _mode=GL_RGBA);

  /**
  Destructor
  */
  virtual ~LightMap() {}

  /// Set the format of the lightmap. GL_RGBA and GL_LUMINANCE_ALPHA are 
  /// supported.
  void set_mode(GLint _mode) { mode_ = _mode; }

  /** Perform the calculation of the lighting terms according to the current 
	  OpenGL-States and the selected format.
  */
  void update();



protected:

  typedef OpenMesh::Vec4f  Color;
  typedef OpenMesh::Vec3f  Normal;

  struct LightSource
  {
    OpenMesh::Vec4f  position;
    Color       ambient, diffuse, specular;
    bool        enabled;
  };



  virtual OpenMesh::Vec4uc light(const Normal& _n) const;

  Color        mat_emission_, mat_ambient_, mat_diffuse_, mat_specular_;
  float        mat_shininess_;
  LightSource  lights_[8];
  Color        ambient_light_;


  GLint mode_;


private:
  
  void  get_GL_params();

  LightMap(const LightMap& _rhs);
  LightMap& operator=(const LightMap& _rhs);
};


//=============================================================================
#endif // PBR_LIGHTMAP_HH defined
//=============================================================================

