/******************************************************************************
                          C++ Interface: Timer
                            -------------------
    description       :
    begin             : Thu Mar 24 2005
    author            : Martin Marinov, (C) 2005
    email             : marinov@cs.rwth-aachen.de
    copyright         : See LICENSE.txt file that comes with this distribution
 ******************************************************************************/

#ifndef __Timer_h__
#define __Timer_h__

#include <GL/glut.h>
#include "defs.h"

class Timer
{
protected:
  uint                                      n_total_frames_;
  uint                                      total_time_;
  uint                                      recent_time_;  
  uint                                      n_scene_vertices_;
  uint                                      n_scene_faces_;
  
public:
  Timer()
  : n_total_frames_(0), recent_time_(0), n_scene_vertices_(0), n_scene_faces_(0)
  {}
  
  void                                      set_scene_complexity(uint _n_scene_vertices, 
                                                                 uint _n_scene_faces)
  { 
    n_scene_vertices_ = _n_scene_vertices; 
    n_scene_faces_ = _n_scene_faces;
  }

/*  inline void                               update()
  {
    n_total_frames_++; 
    uint t = glutGet(GLUT_ELAPSED_TIME);
    if (t - recent_time_ >= 5000)
    {
      double seconds = (t - recent_time_) / 1000.0;
      double fps = n_total_frames_ / seconds;
      double vps = (double)n_total_frames_*(double)n_scene_vertices_/(seconds*1000000.0);
      printf("%d frames in %.3f seconds = %.3f FPS, or %.6fM VPS\n", n_total_frames_, seconds, fps, vps);
      recent_time_ = t;
      n_total_frames_ = 0;
    }
  }*/

  inline bool                               update(double& _fps, double& _mvps, double& _mfps)
  {
    n_total_frames_++; 
    uint t = glutGet(GLUT_ELAPSED_TIME);
    if (t - recent_time_ < 500)
    {//update once every half-second
      return false;
    }
    double seconds = (t - total_time_) / 1000.0;
    
    _fps = n_total_frames_ / seconds;
    _mvps = _fps*n_scene_vertices_/1000000.0;
    _mfps = _fps*n_scene_faces_/1000000.0;
    
    recent_time_ = t;
    return true;
  }
  
  inline void                               reset()
  {
    total_time_ = recent_time_ = glutGet(GLUT_ELAPSED_TIME);
    n_total_frames_ = 0;
  }    
};

#endif//__Timer_h__
