/******************************************************************************
                          C++ Interface: VertexProgram
                            -------------------
    description       :
    begin             : Wed Mar 23 2005
    author            : Martin Marinov, (C) 2005
    email             : marinov@cs.rwth-aachen.de
    copyright         : See LICENSE.txt file that comes with this distribution
 ******************************************************************************/

#ifndef __VertexProgram_h__
#define __VertexProgram_h__

#include <vector>
#include <string>

#ifdef WIN32
  #include <windows.h>
  #include <GL/glew.h>
#else
  #include <GL/glew.h>
#endif

#include "defs.h"

class VertexProgram
{
public:
  typedef std::vector<std::string>        StringArray;
  typedef std::vector<int>                IndexArray;
  
private:
  GLhandleARB                             vsh_hnd_;
  GLhandleARB                             program_hnd_;
  bool                                    active_;
  std::string                             source_;
  StringArray                             bindings_;
  
public:
  VertexProgram();
  ~VertexProgram();
  
  bool                                    load_source(const char* _file_name);
  void                                    push_front_source(const StringArray& _lines); 
  /** Specifies a set of attribute bindings. _bindings.size() must be <= maxium allowed attributes number.
      NOTE: The attribute index i+1 is mapped to _bindings[i] , as the index 0 is reserved for the 
      standard attribute gl_Vertex.
  */
  void                                    set_bindings(const StringArray& _bindings);
  bool                                    compile_and_link();  
  void                                    query_attrib_locations(const StringArray& _names, 
                                                                 IndexArray& _locations) const;
  int                                     query_attrib_location(const std::string& _name) const;
  void                                    query_uniform_locations(const StringArray& _names, 
                                                                 IndexArray& _locations) const;
  int                                     query_uniform_location(const std::string& _name) const;
                                                                 
  
  inline void                             start()
  { 
    OGL_CHECK(::glUseProgramObjectARB(program_hnd_)); 
    active_ = true;
  }
  inline void                             stop()
  { 
    OGL_CHECK(::glUseProgramObjectARB(0)); 
    active_ = false;
  }
  inline bool                             active() const
  { return active_; }
  
  void                                    dump_source() const;
};

#endif//__VertexProgram_h__
