/******************************************************************************
                         C++ Interface: Definitions
                            -------------------
    description       :
    begin             : Wed Mar 23 2005
    author            : Martin Marinov, (C) 2005
    email             : marinov@cs.rwth-aachen.de
    copyright         : See LICENSE.txt file that comes with this distribution
 ******************************************************************************/

#ifndef __defs_h__
#define __defs_h__

#include <assert.h>

#undef min
#undef max
#ifdef WIN32
#define __inline__ __forceinline
#endif
void check_ogl_error();

#ifdef DEBUG
  #define OGL_CHECK(EXPR)     EXPR; check_ogl_error();
  #define CHECK(EXPR)         assert(EXPR);
  #define CHECK_FALSE         assert(false);
#else
  #define OGL_CHECK(EXPR)     EXPR;
  #define CHECK(EXPR)         
  #define CHECK_FALSE         
#endif    


template <bool Expr> struct CompileAssertT;
template <> struct CompileAssertT<true> {};

#define CHECK_STATIC(EXPR)    CompileAssertT<(EXPR)>();

typedef unsigned int uint;

namespace htf
{
template <class _T0, class _T1>
inline void set_min(_T0& _a, const _T1& _b)
{
  if (_b < _a)
  {
    _a = (_T0)_b;
  }
}

template <class _T0, class _T1>
inline void set_max(_T0& _a, const _T1& _b)
{
  if ( _b > _a )
  {
    _a = (_T0)_b;
  }
}

template <class _T>
const _T& min(const _T& _a, const _T& _b)
{ return _a < _b ? _a : _b; }

template <class _T>
const _T& max(const _T& _a, const _T& _b)
{ return _a < _b ? _b : _a; }

template <class _T>
_T sqr(const _T& _a)
{ return _a*_a; }

template <class _T>
__inline__ void swap(_T& _a, _T& _b)
{
  _T c = _a;
  _a = _b;
  _b = c;
}

template <class _T>
__inline__ int                            sign(_T _a)
{ return _a < 0 ? -1 : 1; }
};//namespace htf

#endif//__defs_h__
